% comparison implementation of Yu & Ballard (2008) 
% translation model from Brown et al. (1994), "IBM Model I"
%
% described in "Using speakers' referential intentions to model early
% cross-situational word learning." Psych Science (submitted).
%
% note: this implementation optimizes P(objects | words) as described in Yu
% & Ballard (2007).
%
% Before running this, you might want to run "clear all" to start with
% a fresh workspace.

% Make it possible to run helper functions and access data files,
% by adding them to the path.
addpath([pwd '/data']);
addpath([pwd '/helper_functions']);
addpath([pwd '/baseline_functions']);

% Load the data files.
load corpus.mat
load world.mat
load gold_standard.mat

%% Plot the object frequencies
[sorted_objects, sorted_object_i] = sort(world.objects_freq,'descend');
figure(1)
bar(sorted_objects);xlabel('word');ylabel('frequency')
set(gca,'XTick',1:world.num_objects) 
xticklabels(world.objects_key(sorted_object_i))


%% Exercise: plot the word frequencies (top 30)


%%
num_iter = 5; % number of EM steps

%% add a null to the list of objects in each situation
world.num_objects = world.num_objects + 1;
world.objects_key{end+1} = 'NULL';
null_obj = world.num_objects;

for i = 1:length(corpus)
  corpus(i).objects(end+1) = null_obj;
end

%% run the EM algorithm
thetas{1} = ones(world.num_objects,world.num_words) ./ world.num_words;

fprintf('*** running EM ***\niter\tchange\n');
for i = 2:num_iter
  estimated_counts{i} = computeCounts(thetas{i-1},corpus);
  thetas{i} = computeAssociations(estimated_counts{i},corpus,world);
  fprintf('%d\t%2.2f\n',i-1,nansum(nansum(abs(thetas{i}-thetas{i-1}))));
end

%% Evaluate the results

yu_model_thetas = thetas{end}(:,1:end-1)'; % remove nulls
[p,r,f,yu_lex] = consolidateMatrix(yu_model_thetas,gold_standard,world);
figure(4);
evalLexicon(yu_lex,gold_standard,world);